﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace GetSymbolInfo
{
    public partial class Form1 : Form
    {
        private WinWrap.Basic.BasicNoUIObj basicNoUIObj_;
        private string filename_; // full path

        public Form1()
        {
            InitializeComponent();
            listBoxMacros.Items.AddRange(Macros().ToArray());
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            basicNoUIObj_ = new WinWrap.Basic.BasicNoUIObj();
            basicNoUIObj_.Secret = new Guid("{00000000-0000-0000-0000-000000000000}");
            basicNoUIObj_.Initialize();
            basicNoUIObj_.AttachToForm(this, WinWrap.Basic.ManageConstants.All);
            labelVersion.Text = basicNoUIObj_.Query("GetSystemInfo")["Version"].ToString();
        }

        private void listBoxMacros_SelectedIndexChanged(object sender, EventArgs e)
        {
            var selectedItem = listBoxMacros.SelectedIndex;
            if (selectedItem != -1)
            {
                // Read Macro
                listBoxGlobalMembers.Items.Clear();
                filename_ = $"{MacrosDir()}\\{listBoxMacros.SelectedItem.ToString()}";
                textBoxMacro.Text = File.ReadAllText(filename_);

                // Get Global Members and Types
                string expr = "Global";
                int index = 0;
                string jsonresult = basicNoUIObj_.SymbolInfo(filename_, expr, index, false);
                textBoxSymbolInfo.Text = jsonresult;
                dynamic result = WinWrap.Basic.Util.ParseJson(jsonresult);
                listBoxGlobalMembers.Items.Clear();
                listBoxGlobalMembers.Items.AddRange((string[])result.members(false));
                listBoxGlobalMembers.Items.AddRange((string[])result.types(false));
                ShowSymbolInfoCommand(filename_, expr, index);
            }
        }

        private void textBoxMacro_Click(object sender, EventArgs e)
        {
            string expr = "";
            int index = textBoxMacro.SelectionStart;
            string jsonresult = basicNoUIObj_.SymbolInfo(filename_, expr, index, false);
            textBoxSymbolInfo.Text = jsonresult;
            dynamic result = WinWrap.Basic.Util.ParseJson(jsonresult);
            if ((bool)result.expr_at(true))
            {
                textBoxMacro.SelectionStart = result.expr_at[0];
                textBoxMacro.SelectionLength = result.expr_at[3];
                textBoxMacro.Select();
            }
            listBoxGlobalMembers.ClearSelected();
            ShowSymbolInfoCommand(filename_, expr, index);
        }

        private void listBoxGlobalMembers_SelectedIndexChanged(object sender, EventArgs e)
        {
            var selectedItem = listBoxGlobalMembers.SelectedIndex;
            if (selectedItem != -1)
            {
                int index = 0;
                string membertype = listBoxGlobalMembers.SelectedItem.ToString();
                Regex regex = new Regex("^[^( ]+");
                Match match = regex.Match(membertype);
                string expr = match.Value;
                textBoxSymbolInfo.Text = basicNoUIObj_.SymbolInfo(filename_, expr, index, false);
                dynamic result = WinWrap.Basic.Util.ParseJson(textBoxSymbolInfo.Text);
                textBoxMacro.SelectionStart = textBoxMacro.TextLength - 1;
                textBoxMacro.Select();
                textBoxMacro.ScrollToCaret();
                textBoxMacro.SelectionStart = result.symbol_at[0];
                textBoxMacro.SelectionLength = result.symbol_at[3];
                textBoxMacro.Select();
                textBoxMacro.ScrollToCaret();
                ShowSymbolInfoCommand(filename_, expr, index);
            }
        }

        private List<string> Macros()
        {
            List<string> macros = new List<string>();
            List<string> paths = Directory.GetFiles(MacrosDir(), "*.*").ToList();
            macros.AddRange(paths.ConvertAll(path => Path.GetFileName(path)));
            macros.Sort();
            return macros;
        }

        private string MacrosDir()
        {
            string rootdir = Path.GetDirectoryName(typeof(Form1).Assembly.Location);
            return $"{rootdir}\\Macros";
        }

        private void ShowSymbolInfoCommand(string filepath, string expr, int index)
        {
            string macroname = Path.GetFileName(filepath);
            textBoxCommand.Text = $"basicNoUIObj_.SymbolInfo(\"...\\{macroname}\", \"{expr}\", " +
                $"{index.ToString()}, false)";
        }
    }
}
